// -*-C++-*-

// --8<--8<--8<--8<--
//
// Copyright (C) 2011 Smithsonian Astrophysical Observatory
//
// This file is part of bpmanip
//
// bpmanip is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or (at
// your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
// -->8-->8-->8-->8--

#ifndef BPMANIP_BINARYPIPE_H
#define BPMANIP_BINARYPIPE_H

#include <list>
#include <string>

#include <bpipe/bpipe.h>
#include <bpipe/bpipe_axaf.h>
#include <bpipe/datatypes.h>

#include "Fields.h"

typedef std::vector< std::string > Container;
typedef Container::const_iterator ConIter;

class BinaryPipe {

    public:
        // Need to get this to work with default arg and reference!
        //
        BinaryPipe ( std::string& input, std::string& output );

        virtual ~BinaryPipe ();

        void WriteHeader         ();
        size_t ReadDataPackets   (size_t n );
        void MapAlloc            (size_t n );
        void GetDataFieldNames   ( Container& fieldnames );
        void DeleteDataField     ( const std::string& fieldname );
        void AlterCore           ( Fields& fields );
        void CopyPackets         ();

    protected:
        BPipe*       m_bpipe;       // Used by libbpipe library, BPipe and
        BPipeOutput* m_bpipe_out;   //    BPipeOutput are library typedefs.

        // Must use void* members for libbpipe library compatability
        //
        void*      m_packet_data_ptr;

        static const size_t buffer_size = 1000;


};

#endif // ! BINARYPIPE_H_INCLUDED
