-- Use Chandra aspect solutions file for driving telescope jitter.

local tonumber = tonumber
local assert = assert

local pi = require('math').pi

local tables = require( 'saotrace.suplib.tables' )

local raygen = require( 'saotrace.raygen' )
local jitter = require( 'saotrace.raygen.jitter' );

local vobj = require( 'saotrace.raygen.validate').vobj:new()

local M = {}
setfenv( 1, M )

function dither_asol( ... )

   local ok, args = vobj:validate( { { name = 'file',
				       type = 'string' } },
				   ... )
   assert( ok, args )


   local fits = raygen.fits.table:new( args.file )

   local function angle( name )
      return tonumber(fits:keyword( name ))
   end

   local RA   = angle('RA_NOM')
   local Dec  = angle('DEC_NOM')
   local Roll = angle('ROLL_NOM')

   raygen.jitter.file{ name = "asol",
		       file = args.file,

		       quat_col = 'q_att',

		       optical_axis = {

			  {  angle = { Roll, 'degree' }, axis = {  1,  0,  0 } },
			  {  angle = { Dec,  'degree' }, axis = {  0, -1,  0 } },
			  {  angle = { RA,   'degree' }, axis = {  0,  0,  1 } },

		       },

		       coord_xfrm = {
			  -- transformations applied to input coords to get raytrace coords
			  x = { 0,  -1,  0 },
			  y = { 0,   0, -1 },
			  z = { -1,  0,  0 }
		       },

		    }
   raygen.jitter.roll( { Roll, 'degree' } )

   return { ra = RA, dec = Dec, roll = Roll }

end

function lissajous( args )

   raygen.jitter.lissajous( 'lj', args )

end


return { dither_asol = dither_asol,
	 lissajous = lissajous,
	 roll = raygen.jitter.roll
      }
