local pairs = pairs
local print = print
local select = select
local require = require
local system = require( 'saotrace.raygen.system' )
local _G = _G

module( ... )

stash = {}

local intrinsics = { 'extent',
		     'spectrum',
		     'jitter',
		     'ea',
		     'source',
		  }

function mock_func( mod, func )

   return function( ... )

	     stash[#stash+1] = { mod = mod,
				 func = func,
				 ... }
	  end

end


-- create mock functions for the intrinsics if the intrinsics don't exist.
-- this is used during pure Lua testing, when the actual raygen intrinsics
-- aren't loaded

_M.source = {

   finish = function ( ... )

	       system.inc_nsources()
	       stash[#stash+1] = { mod = 'source',
				   func = 'finish',
				 ... }
	    end
}

for _, v in pairs( intrinsics ) do

   local mod = require( _PACKAGE .. v )

   if not _M[v] then
      _M[v] = {}
   end

   for func in pairs( mod.vspec ) do

      if not _M[v][func] then

	 _M[v][func] = mock_func( v, func )

      end
   end
end

