require 'lunatest'

vobj = require('va').vobj

convert_angle = require( 'saotrace.raygen.units' ).convert_angle
idist = require( 'saotrace.raygen.validate.idist' )

function validate( ... )
   return idist.validate( vobj, ... )
end

function test_uniform1D ()

   local ok, args = validate( {
				      x = { 'uniform' },
				      y = { 'uniform' },
				   })
   assert_true(ok, args)
   assert_equal( 'uniform', args.x[1] );
   assert_equal( 'uniform', args.y[1] );


   local ok, args = validate( {
				      x = { 'uniform', 2 },
				      y = { 'uniform', 3 },
				   })
   assert_false(ok, args)

end

function test_gaussian1D ()

   local ok, args = validate( {
				      x = { 'gaussian' },
				      y = { 'gaussian' },
				   })
   assert_false(ok, args)

   local ok, args = validate( {
				 x = { 'gaussian', sigma = { 2, 'arcmin' } },
				 y = { 'gaussian', sigma = '3 arcmin' },
				   })
   assert_true(ok, args)
   assert_equal( 0, args.x.mean[1] );
   assert_equal( 2, args.x.sigma[1] );

   assert_equal( 0, args.y.mean[1] );

   -- args.y is in units of 3 arcmin (yes!)
   assert_equal( 1, args.y.sigma[1] );
   local ok, val = convert_angle( 1, args.y.sigma[2], 'arcminute' )
   assert_equal( 3, val[1], .01 )


   local ok, args = validate( {
				      x = { 'gaussian', sigma = '2 arcmin', mean = '1 arcmin' },
				      y = { 'gaussian', sigma = {4, 'arcmin'}, mean = {2, 'arcmin'} },
				   })
   assert_true(ok, args)
   assert_equal( 1, args.x.mean[1] );

   -- args.x is in units of 2 arcmin
   assert_equal( 1, args.x.sigma[1] );
   local ok, val = convert_angle( 1, args.x.sigma[2], 'arcminute' )
   assert_equal( 2, val[1], .01 )

   assert_equal( 2, args.y.mean[1] );
   assert_equal( 4, args.y.sigma[1] );

end

function test_uniform2D ()

   local ok, args = validate( { 'uniform' }  )
   assert_true(ok, args)

   local ok, args = validate( { 'uniform', 3 }  )
   assert_false(ok, args)

end

function test_gaussian2D ()

   local ok, args = validate( { 'gaussian' }  )
   assert_false(ok, args)

   local ok, args = validate( { 'gaussian', sigma = { 2, 'arcsec' } }  )
   assert_true(ok, args)
   assert_equal( 0, args.x[1] );
   assert_equal( 0, args.y[1] );
   assert_equal( 2, args.sigma[1] );

   local ok, args = validate( { 'gaussian', x = { 2, 'arcsec' },
				            y = '3 arcsec',
				sigma = { 22, 'arcsec' } }  )
   assert_true(ok, args)
   assert_equal( 2, args.x[1] );

   -- args.y is in units of 3 arcsec (yes!)
   assert_equal( 1, args.y[1] );
   local ok, val = convert_angle( 1, args.y[2], 'arcsecond' )
   assert_equal( 3, val[1], .01 )

   assert_equal( 22, args.sigma[1] );

end


lunatest.run()
