require 'lunatest'
require 'deepcompare'

local raygen = require( 'saotrace.raygen' )

create = require( 'saotrace.raygen.source').create

function test_defaults ()

   local ok, args = pcall( create )

   assert_false( ok );
   assert_match( '%(name%): missing', args  )

   local ok, src = pcall( create, 'test' )

   assert_true( ok, src );
   assert_equal( 'test', src.name )


   local default

   default = raygen.extent.default{ name = src.name }

   assert_equal( default.type, src.extent.type, "extent type")

   assert_true( deepcompare( default.args,
			      src.extent.args ),
		'default extent arguments'
	     )

   default = raygen.spectrum.default{ name = src.name }

   assert_equal( default.type, src.spectrum[1].type, "spectrum type")

   assert_true( deepcompare( default.args,
			     src.spectrum[1].args ),
		'default spectrum arguments'
	     )

end

function test_extent ()

   local pos = raygen.extent.point{ name = 'tp' }

   local ok, src = pcall( create, 'ts', { extent = pos } )

   assert_true( ok, src );
   assert_equal( 'ts', src.name )


   assert_true( deepcompare( pos, src.extent ), 'source extent')

end

function test_spectrum ()

   local s1 = raygen.spectrum.mono{ name = 's1', energy = 2 }
   local s2 = raygen.spectrum.flat{ name = 's2', emin = 1, emax = 10 }

   local ok, src = pcall( create, 'ts', { spectrum = { s1, s2 } } )

   assert_true( ok, src );
   assert_equal( 'ts', src.name )

   assert_true( deepcompare( s1, src.spectrum[1] ), 'source spectrum 1')
   assert_true( deepcompare( s2, src.spectrum[2] ), 'source spectrum 2')

end

lunatest.run()
