// -*-C++-*-
#ifndef JITTER_TABLE_H
#define JITTER_TABLE_H

#include <string>
#include <vector>

#include <Exception/Exception.h>
#include <tracefctxx/TraceFct.h>

#include "Jitter/Common.h"
#include "Jitter/Base.h"
#include "Jitter/File.h"


namespace Jitter {

    class Table : public Base {

    public:

	enum OutOfBounds {
	    OOB_Zero,
	    OOB_Throw
	};


	OutOfBounds out_of_bounds;

	Point* points;

	// input rotations are additive rather than absolute
	bool   rotation_is_additive;

	// location of the optical axis in input coordinates.
	Eigen::Quaterniond optical_axis;

	// origin of input rotations in input coordinates
	Eigen::Quaterniond input_origin;


	Table( ) : points( NULL ), _size(0) {}

	Table( size_t n ) {

	    _size = n;

	    points = new Point[n];
	}

	Table( string& name ) :
	    Base(name),
	    points(NULL),
	    rotation_is_additive(false),
	    optical_axis(Eigen::Quaterniond::Identity()),
	    input_origin(Eigen::Quaterniond::Identity()),
	    _size(0),
	    _oob( OOB_Throw )
	{
	}

	Table( const char* name ) :
	    Base(name),
	    points(NULL),
	    rotation_is_additive(false),
	    optical_axis(Eigen::Quaterniond::Identity()),
	    input_origin(Eigen::Quaterniond::Identity()),
	    _size(0),
	    _oob( OOB_Throw )
	{
	}

	void read( TableFile::Base* file);

	~Table() {
	    if ( points )
		delete[] points;
	}

	size_t size() { return _size; }

	void jitter( double time, Eigen::Quaterniond& q );

    private:

	struct Rotation {
	    size_t idx;

	    // absolute quaternion rotation from fiducial
	    Eigen::Quaterniond q;

	    Rotation() : idx(0), q(Eigen::Quaterniond::Identity()) {}

	};

	size_t _size;
	Rotation _last;
	OutOfBounds _oob;

    };

}

#endif // ! JITTER_TABLE_H
