module( ..., package.seeall )

local table = require( 'table' )
local os = require( 'os' )
local srcdir = os.getenv('srcdir') or '.'
local units = require('saotrace.raygen.units')
local convert = units.convert

local simple = require( 'saotrace.raygen.source.simple' )

function setup ()
   _G.setup()
   saotrace.raygen.intrinsic.stash = {}
end

function test_point( )

   simple.point{ spectrum = 1.49, position = { theta = 2, phi = 9 } }
   local stash = saotrace.raygen.intrinsic.stash

   assert_equal( 'point1', stash[1][1].name )

   local position = stash[2][1].position

   assert_equal( 2, convert( position.theta, nil, units.angular.arcminute)[1] )
   assert_equal( 9, convert( position.phi, nil, units.angular.degree )[1] )
   assert_equal( -1e30, convert( position.z, nil, units.linear.mm )[1] )

   local spectrum = stash[3]

   assert_equal( 1.49, spectrum[1].energy )
   assert_equal( 'mono', spectrum.func )

end

function test_disk( )

   simple.disk{ spectrum = 1.49, radius = { 5, 'arcsec' },
		position = { theta = 2, phi = 9}
	     }
   local stash = saotrace.raygen.intrinsic.stash

   local pars = stash[2][1]

   assert_equal( 5, convert( pars.radius, nil, units.angular.arcsecond)[1] )

   assert_equal( 2, convert( pars.position.theta, nil, units.angular.arcminute)[1] )
   assert_equal( 9, convert( pars.position.phi, nil, units.angular.degree )[1] )
   assert_equal( -1e30, convert( pars.position.z, nil, units.linear.mm )[1] )

end

function test_load( )

   local srcfile = table.concat( { srcdir, 'data', 'sources.xml'}, '/' )

   simple.load_sources( srcfile )

end
